import argparse
import logging
import sys
import traceback


from twisted.internet import reactor
from twisted.internet.defer import inlineCallbacks
import yaml
import zookeeper

from aiki.introspect import get_zk_client_connector


LOG_LEVELS = dict(
    CRITICAL=logging.CRITICAL,
    ERROR=logging.ERROR,
    WARNING=logging.WARNING,
    INFO=logging.INFO,
    DEBUG=logging.DEBUG)


def make_arg_parser(*args, **kwargs):
    parser = argparse.ArgumentParser(*args, **kwargs)
    parser.add_argument(
        "-e", "--environment", default=None, help="Environment to act upon (otherwise uses default)", metavar="ENVIRONMENT")
    parser.add_argument(
        "--loglevel", default=None, choices=LOG_LEVELS, help="Log level",
        metavar="CRITICAL|ERROR|WARNING|INFO|DEBUG")
    parser.add_argument("--verbose", "-v", default=False, action="store_true", help="Enable verbose logging")
    return parser


def setup_logging(options):
    options.loglevel = LOG_LEVELS.get(options.loglevel)
    if options.verbose:
        if options.loglevel is None:
            options.loglevel = logging.DEBUG
    else:
        # Avoid potentially voluminous ZK debug logging 
        zookeeper.set_debug_level(0)
        if options.loglevel is None:
            options.loglevel = logging.INFO

    log_options = {
        "level": options.loglevel,
        "format": "%(asctime)s %(name)s:%(levelname)s %(message)s"
        }
    logging.basicConfig(**log_options)


def run_command(command, options):
    """Connect to ZK, either from admin machine or on a Juju machine, then run command"""
    # Introspect to find the right client connector; this will be
    # called, then yielded upon, to get a ZK client once async code is
    # entered
    try:
        connector = get_zk_client_connector(options)
    except Exception, e:
        print >> sys.stderr, e
        sys.exit(1)

    exit_code = [0]
    reactor.callWhenRunning(_run_command_in_reactor, command, exit_code, connector, options)
    reactor.run()
    sys.exit(exit_code[0])


@inlineCallbacks
def _run_command_in_reactor(command, exit_code, connector, options):
    try:
        yield _wrapped_command(command, exit_code, connector, options)
    except Exception, e:
        exit_code[0] = 1
        if options.verbose:
            traceback.print_exc()  # Writes to stderr
        else:
            print >> sys.stderr, e
    finally:
        reactor.stop()


@inlineCallbacks
def _wrapped_command(command, result, connector, options):
    client = yield connector()
    try:
        yield command(result, client, options)
    finally:
        yield client.close()


def parse_relation_settings(args):
    """Given args, return a list of the remaining args along with settings"""
    settings = {}
    kv_iter = iter(args)
    remaining_args = []
    for kv in kv_iter:
        if "=" not in kv:
            remaining_args = [kv]
            break
        k, v = kv.split("=", 1)
        if v.startswith("@"):
            filename = v[1:]
            with open(filename, "r") as f:
                v = f.read()
        settings[k] = yaml.safe_load(v)
    remaining_args.extend(kv_iter)
    return settings, remaining_args
